package animation;

import models.AnimatedModel;
import models.BoneInfo;
import models.Node;
import org.lwjgl.PointerBuffer;
import org.lwjgl.assimp.AIAnimation;
import org.lwjgl.assimp.AINode;
import org.lwjgl.assimp.AINodeAnim;
import org.lwjgl.assimp.AIScene;
import models.skeleton.Skeleton;
import utils.MathUtill;

import java.util.LinkedList;
import java.util.Objects;

public class Animation {

    private double ticksForSecond;
    private double duration;
    private Node rootNode;
    private LinkedList<Bone> bones;
    private AIAnimation aiAnimation;
    private Skeleton skeleton;

    public Animation(AnimatedModel model, AIScene aiScene, int animationIndex) {
        if (aiScene == null)
            return;

        rootNode = new Node();
        aiAnimation = AIAnimation.create(aiScene.mAnimations().get(animationIndex));
        ticksForSecond = aiAnimation.mTicksPerSecond();
        duration = aiAnimation.mDuration();
        processNodes(rootNode, Objects.requireNonNull(aiScene.mRootNode()));
        readMissingBones(model);
    }

    public Node processNodes(Node node, AINode aiParentNode){
        //setting node with data from aiParentNode
        node.setName(aiParentNode.mName().dataString());
        node.setTransformation( MathUtill.toMat4Matrix(aiParentNode.mTransformation()));
        node.setChildrenCount(aiParentNode.mNumChildren());

        for (int i = 0; i < aiParentNode.mNumChildren(); i++){
            Node childNodeData = new Node();
            AINode childNode = AINode.create(aiParentNode.mChildren().get(i));

            processNodes(childNodeData,childNode);
            node.getChildren().addLast(childNodeData);
        }
        return node;
    }

    private void readMissingBones(AnimatedModel model){
        PointerBuffer aiChannels = aiAnimation.mChannels();
        int boneCount = model.getBoneInfoHashMap().size();
        for (int i = 0; i< aiAnimation.mNumChannels();i++){
            AINodeAnim aiNodeAnim = AINodeAnim.create(aiChannels.get(i));
            String boneName = aiNodeAnim.mNodeName().dataString();
            if (model.getBoneInfoHashMap().get(boneName) == null){
                BoneInfo boneInfo = new BoneInfo(boneCount,null);
                model.getBoneInfoHashMap().put(aiNodeAnim.mNodeName().dataString(),boneInfo);
                boneCount++;
            }
        }
    }

    public LinkedList<Bone> getBones() {
        return bones;
    }

    public void setBones(LinkedList<Bone> bones) {
        this.bones = bones;
    }

    public double getTicksForSecond() {
        return ticksForSecond;
    }

    public void setTicksForSecond(double ticksForSecond) {
        this.ticksForSecond = ticksForSecond;
    }

    public double getDuration() {
        return duration;
    }

    public void setDuration(double duration) {
        this.duration = duration;
    }

    public Node getRootNode() {
        return rootNode;
    }

    public void setRootNode(Node rootNode) {
        this.rootNode = rootNode;
    }

    public AIAnimation getAiAnimation() {
        return aiAnimation;
    }
}
