package models;

import utils.MathUtill;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class MeshData {
    List<Vertex> vertices;
    List<Integer> indices;

    public MeshData(List<Integer> indices, List<Vertex> vertices) {
        this.vertices = vertices;
        this.indices = indices;
    }

    public float[] getVertexAttributes() {
        List<Float> floatVertices = new ArrayList<>();
        Vertex vertex;

        for (int i = 0; i < vertices.size(); i++) {
            vertex = vertices.get(i);

            //postions to vertexArray
            floatVertices.add((float) vertex.getPosition().getX());
            floatVertices.add((float) vertex.getPosition().getY());
            floatVertices.add((float) vertex.getPosition().getZ());

            //normals to vertexArray
            floatVertices.add((float) vertex.getNormal().getX());
            floatVertices.add((float) vertex.getNormal().getY());
            floatVertices.add((float) vertex.getNormal().getZ());

            //bones to vertexArray
            int[] bones = vertex.getVertexBoneInfo().getBoneIds();
            floatVertices.add((float) bones[0]);
            floatVertices.add((float) bones[1]);
            floatVertices.add((float) bones[2]);
            floatVertices.add((float) bones[3]);

            // weights to vertexArray
            float[] weights = vertex.getVertexBoneInfo().getWeights();
            floatVertices.add(weights[0]);
            floatVertices.add(weights[1]);
            floatVertices.add(weights[2]);
            floatVertices.add(weights[3]);

            //normals
//            floatVertices.add((float)vertex.getNormal().getX());
//            floatVertices.add((float)vertex.getNormal().getY());
//            floatVertices.add((float)vertex.getNormal().getZ());
        }
        return MathUtill.getFloatArray(floatVertices);
    }

    public float[] getVertexAttributesSingleBone(boolean biggestBone, int boneIndex) {
        List<Float> floatVerticies = new ArrayList<>();
        Vertex vertex;

        for (int i = 0; i < vertices.size(); i++) {
            vertex = vertices.get(i);

            //postions to vertexArray
            floatVerticies.add((float) vertex.getPosition().getX());
            floatVerticies.add((float) vertex.getPosition().getY());
            floatVerticies.add((float) vertex.getPosition().getZ());

            // finding max weight
            float[] weights = vertex.getVertexBoneInfo().getWeights();
            int maxWeight = MathUtill.findMaxIndexOfArray(weights);

            //bones to vertexArray
            int[] bones = vertex.getVertexBoneInfo().getBoneIds();
            if (biggestBone)
                floatVerticies.add((float) bones[maxWeight]);
            else if (!biggestBone && boneIndex < 4)
                floatVerticies.add((float) bones[boneIndex]);
            else return null;

            //normals
            floatVerticies.add((float)vertex.getNormal().getX());
            floatVerticies.add((float)vertex.getNormal().getY());
            floatVerticies.add((float)vertex.getNormal().getZ());


        }
        return MathUtill.getFloatArray(floatVerticies);
    }
    public List<Integer> getIndices() {
        return indices;
    }

}
