package rendering;

import animation.Animation;
import animation.Animator;
import models.AnimatedModel;
import assimp.loaders.MeshLoader;
import org.lwjgl.BufferUtils;
import org.lwjgl.assimp.*;
import org.lwjgl.glfw.*;
import models.skeleton.Skeleton;
import transforms.*;
import utils.*;

import java.io.File;
import java.lang.Math;
import java.nio.DoubleBuffer;
import java.nio.FloatBuffer;
import java.util.*;

import static org.lwjgl.glfw.GLFW.*;
import static org.lwjgl.opengl.GL11.glClearColor;
import static org.lwjgl.opengl.GL20.*;


public class Renderer extends AbstractRenderer {

    double ox, oy, startTimeMilis;
    boolean mouseButton1 = false;

    OGLBuffers buffers2, buffers;

    int shaderProgramS, locMatProjS, locMatViewS, bonesLocationS, viewVectorS,
            shaderProgramB, locMatProjB, locMatViewB, bonesLocationB, viewVectorB,
            shaderProgramM, locMatProjM, locMatViewM, bonesLocationM, viewVectorM;

    AnimatedModel model;
    OGLBuffers bufferS;
    List<OGLBuffers> buffersListM = new ArrayList<>();

    Camera cam = new Camera();
    Mat4 proj = new Mat4PerspRH(Math.PI / 4, 1, 0.01, 1000.0);
    Mat4 ortho = new Mat4OrthoRH(20, 30, 0.01, 1000.0);
    boolean perspective = true;
    boolean anim = true, blending = true;

    AIScene aiScene;
    AIAnimation aiAnimation;
    AINode rootNode;
    Animation animation;
    Animator animator;
    Skeleton skeleton;
    long startUpTime, lastTime;
    double speed = 1;
    Vec3D view;
    int frame = 0;


    private GLFWKeyCallback keyCallback = new GLFWKeyCallback() {
        @Override
        public void invoke(long window, int key, int scancode, int action, int mods) {
            if (key == GLFW_KEY_ESCAPE && action == GLFW_RELEASE)
                glfwSetWindowShouldClose(window, true); // We will detect this in the rendering loop
            if (action == GLFW_PRESS || action == GLFW_REPEAT) {
                switch (key) {
                    case GLFW_KEY_W:
                        cam = cam.forward(5);
                        break;
                    case GLFW_KEY_D:
                        cam = cam.right(5);
                        break;
                    case GLFW_KEY_S:
                        cam = cam.backward(5);
                        break;
                    case GLFW_KEY_A:
                        cam = cam.left(5);
                        break;
                    case GLFW_KEY_LEFT_CONTROL:
                        cam = cam.down(5);
                        break;
                    case GLFW_KEY_LEFT_SHIFT:
                        cam = cam.up(5);
                        break;
                    case GLFW_KEY_SPACE:
                        cam = cam.withFirstPerson(!cam.getFirstPerson());
                        if (!cam.getFirstPerson())
                            cam = cam.withPosition(new Vec3D(0, 0, 0));
                        break;
                    case GLFW_KEY_R:
                        cam = cam.mulRadius(0.9f);
                        break;
                    case GLFW_KEY_F:
                        cam = cam.mulRadius(1.1f);
                        break;
                    case GLFW_KEY_P:
                        perspective = !perspective;
                        break;
                    case GLFW_KEY_M:
                        anim = !anim;
                        break;
                    case GLFW_KEY_V:
                        blending = !blending;
                        break;
                    case GLFW_KEY_B:
                        speed *= 0.9;
                        if (speed < 0.001) speed = 0.001;
                        break;
                    case GLFW_KEY_N:
                        speed *= 1.1;
                        if (speed > 100) speed = 100;
                        break;

                }
            }
        }
    };

    private GLFWWindowSizeCallback wsCallback = new GLFWWindowSizeCallback() {
        @Override
        public void invoke(long window, int w, int h) {
            if (w > 0 && h > 0 &&
                    (w != width || h != height)) {
                width = w;
                height = h;
                proj = new Mat4PerspRH(Math.PI / 4, height / (double) width, 0.01, 500.0);
                ortho = new Mat4OrthoRH(5, 5*height / (double) width, 0.01, 500.0);
            }
        }
    };

    private GLFWMouseButtonCallback mbCallback = new GLFWMouseButtonCallback() {
        @Override
        public void invoke(long window, int button, int action, int mods) {
            mouseButton1 = glfwGetMouseButton(window, GLFW_MOUSE_BUTTON_1) == GLFW_PRESS;

            if (button == GLFW_MOUSE_BUTTON_1 && action == GLFW_PRESS) {
                mouseButton1 = true;
                DoubleBuffer xBuffer = BufferUtils.createDoubleBuffer(1);
                DoubleBuffer yBuffer = BufferUtils.createDoubleBuffer(1);
                glfwGetCursorPos(window, xBuffer, yBuffer);
                ox = xBuffer.get(0);
                oy = yBuffer.get(0);
            }

            if (button == GLFW_MOUSE_BUTTON_1 && action == GLFW_RELEASE) {
                mouseButton1 = false;
                DoubleBuffer xBuffer = BufferUtils.createDoubleBuffer(1);
                DoubleBuffer yBuffer = BufferUtils.createDoubleBuffer(1);
                glfwGetCursorPos(window, xBuffer, yBuffer);
                double x = xBuffer.get(0);
                double y = yBuffer.get(0);
                cam = cam.addAzimuth((double) Math.PI * (ox - x) / width)
                        .addZenith((double) Math.PI * (oy - y) / width);
                ox = x;
                oy = y;
            }
        }
    };

    private GLFWCursorPosCallback cpCallbacknew = new GLFWCursorPosCallback() {
        @Override
        public void invoke(long window, double x, double y) {
            if (mouseButton1) {
                cam = cam.addAzimuth((double) Math.PI * (ox - x) / width)
                        .addZenith((double) Math.PI * (oy - y) / width);
                ox = x;
                oy = y;
            }
        }
    };

    private GLFWScrollCallback scrollCallback = new GLFWScrollCallback() {
        @Override
        public void invoke(long window, double dx, double dy) {
            if (dy < 0)
                cam = cam.mulRadius(0.9f);
            else
                cam = cam.mulRadius(1.1f);

        }
    };

    @Override
    public GLFWKeyCallback getKeyCallback() {
        return keyCallback;
    }

    @Override
    public GLFWWindowSizeCallback getWsCallback() {
        return wsCallback;
    }

    @Override
    public GLFWMouseButtonCallback getMouseCallback() {
        return mbCallback;
    }

    @Override
    public GLFWCursorPosCallback getCursorCallback() {
        return cpCallbacknew;
    }

    @Override
    public GLFWScrollCallback getScrollCallback() {
        return scrollCallback;
    }


    private void createBuffers() {
        OGLBuffers.Attrib[] attributes = {
                new OGLBuffers.Attrib("in_position", 3),
                //new OGLBuffers.Attrib("in_color", 3),
                new OGLBuffers.Attrib("in_normal", 3),
                new OGLBuffers.Attrib("in_boneIDs", 4),
                new OGLBuffers.Attrib("in_weights", 4),
//                new OGLBuffers.Attrib("in_normal", 3)

        };
        try {
            for (int i = 0; i < model.getMeshes().length; i++) {
                float[] verticies = model.getMesh(i).getVertexAttributes();
                int[] indicies = MathUtill.getIntArray(model.getMesh(i).getIndices());
                buffersListM.add(new OGLBuffers(verticies, 14, attributes, indicies));
            }
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
        startUpTime = System.currentTimeMillis();
    }

    private void createBuffersOneBone() {
        OGLBuffers.Attrib[] attributes = {
                new OGLBuffers.Attrib("in_position", 3),
                new OGLBuffers.Attrib("in_boneID", 1),
                new OGLBuffers.Attrib("in_normal", 3)

        };
        try {
            for (int i = 0; i < model.getMeshes().length; i++) {
                float[] verticies = model.getMesh(i).getVertexAttributesSingleBone(true, 2);
                int[] indicies = MathUtill.getIntArray(model.getMesh(i).getIndices());
                buffersListM.add(new OGLBuffers(verticies, 7, attributes, indicies));
            }
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
        startUpTime = System.currentTimeMillis();
    }

    @Override
    public void init() {
        glClearColor(0.2f, 0.2f, 0.2f, 1.0f);

        loadModel();
        createBuffers();
        //toggle skeleton using v button
        createSkeletonBuffers();
        cam = cam.withPosition(new Vec3D(-10, 0, 0))
                .withAzimuth(Math.PI * -0)
                .withZenith(Math.PI * -0)
                .withFirstPerson(false)
                .withRadius(10);
        view = cam.getViewVector();

        shaderProgramM = ShaderUtils.loadProgram("/animation");
        shaderProgramS = ShaderUtils.loadProgram("/skeletDisplay");

        glUseProgram(shaderProgramS);
        viewVectorS = glGetUniformLocation(shaderProgramS, "in_viewVector");
        locMatProjS = glGetUniformLocation(shaderProgramS, "projectionMatrix");
        locMatViewS = glGetUniformLocation(shaderProgramS, "viewMatrix");
        bonesLocationS = glGetUniformLocation(shaderProgramS, "boneTransforms");

        glUseProgram(shaderProgramM);
        viewVectorM = glGetUniformLocation(shaderProgramM, "in_viewVector");
        locMatProjM = glGetUniformLocation(shaderProgramM, "projectionMatrix");
        locMatViewM = glGetUniformLocation(shaderProgramM, "viewMatrix");
        bonesLocationM = glGetUniformLocation(shaderProgramM, "boneTransforms");
        System.out.println(glGetError());
    }

    @Override
    public void display() {
        glViewport(0, 0, width, height);
        long currentTime = System.currentTimeMillis();
        long delta = currentTime - lastTime;
        if (anim) {
            startUpTime += delta * speed;
        }
        double timeInSeconds = ((double)startUpTime / 1000.0);
        lastTime = currentTime;

        glDisable(GL_CULL_FACE);
        glEnable(GL_DEPTH_TEST);
        if (blending) glEnable(GL_BLEND);
        else glDisable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // clear the framebuffer
        glUseProgram(shaderProgramS);
        if (perspective)
            glUniformMatrix4fv(locMatProjS, false,ToFloatArray.convert(proj));
        else
            glUniformMatrix4fv(locMatProjS, false,ToFloatArray.convert(ortho));
        glUniformMatrix4fv(locMatViewS, false,
                ToFloatArray.convert(cam.getViewMatrix()));
        glUniform3f(viewVectorS, (float) view.getX(), (float) view.getY(), (float) view.getZ());

        //matrices to buffer
        //if (frame>20)
        animator.updateAnimation(timeInSeconds, false);
        Mat4[] finalTransformsArray = animator.getFinalTransforms();
        FloatBuffer matrixBuffer = BufferUtils.createFloatBuffer(finalTransformsArray.length * 16);
        for (Mat4 mat4 : finalTransformsArray) {
            matrixBuffer.put(ToFloatArray.convert(mat4));
        }
        matrixBuffer.rewind();

        glUniformMatrix4fv(bonesLocationS, true, matrixBuffer);

        //draw
        drawSkeleton(shaderProgramS);


        glUseProgram(shaderProgramM);
        if (perspective)
            glUniformMatrix4fv(locMatProjM, false,ToFloatArray.convert(proj));
        else
            glUniformMatrix4fv(locMatProjM, false,ToFloatArray.convert(ortho));
        glUniformMatrix4fv(locMatViewM, false,
                ToFloatArray.convert(cam.getViewMatrix()));
        glUniform3f(viewVectorM, (float) view.getX(), (float) view.getY(), (float) view.getY());

        //matrices to buffer
        animator.updateAnimation(timeInSeconds, true);
        finalTransformsArray = animator.getFinalTransforms();
        for (Mat4 mat4 : finalTransformsArray) {
            matrixBuffer.put(ToFloatArray.convert(mat4));
        }
        matrixBuffer.rewind();
        glUniformMatrix4fv(bonesLocationM, true, matrixBuffer);

        //draw
        drawModel(shaderProgramM);
        frame++;
    }

    private void createSkeletonBuffers() {

        try {
            //absolute path needed
            skeleton = animator.calculateSkelet();
            OGLBuffers.Attrib[] attributes = {
                    new OGLBuffers.Attrib("in_position", 3),
                    new OGLBuffers.Attrib("in_boneID", 1)
            };

            bufferS = new OGLBuffers(skeleton.getVertexAttributes(), 4, attributes, skeleton.getIndices());

        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
        for(int i = 0; i < skeleton.getIndices().length; i+=2){
            int i1 = skeleton.getIndices()[i] * 4;
            int i2 = skeleton.getIndices()[i + 1] * 4;
            System.out.println(i + ": " + i1 + ">" + skeleton.getVertexAttributes()[i1 + 3] +
                    ", " + i2 + ">" + skeleton.getVertexAttributes()[i2 + 3]);
        }
        startUpTime = System.currentTimeMillis();
    }

    private void loadModel() {
        try {
            File file;
            file = new File("res\\vampire.glb");
            model = MeshLoader.loadAnimatedModel(file.getPath());
            aiScene = MeshLoader.getAIScene();
            animation = new Animation(model, aiScene, 0);
            animator = new Animator(animation, model);
        } catch (Exception ex) {
            System.err.println(ex.getMessage());
        }
    }

    private void drawModel(int shaderProgram) {
        for (int i = 0; i < buffersListM.size(); i++) {
            buffersListM.get(i).bind(shaderProgram);
            int indiciesCount = model.getMesh(i).getIndices().size();
            buffersListM.get(i).draw(GL_TRIANGLES, shaderProgram, indiciesCount);
            buffersListM.get(i).unbind();
        }
    }

    private void drawSkeleton(int shaderProgram) {
        bufferS.bind(shaderProgram);
        bufferS.draw(GL_LINES, shaderProgram, skeleton.getIndices().length);
        bufferS.unbind();
    }
}

