package utils;

import java.io.File;

import static org.lwjgl.opengl.GL20.*;
public class ShaderProgram {
    private final int programId;
    private int vertexShaderId;
    private int fragmentShaderId;

    public ShaderProgram() throws Exception {
        programId = glCreateProgram();
        File test = new File("./");
        System.out.println(test.getAbsolutePath());
        if (programId == 0){
            throw new Exception("Could not create shader");
        }
    }

    public void createVertexShader(String shaderCode) throws Exception{
        try {
            vertexShaderId = createShader(shaderCode, GL_VERTEX_SHADER);
        }catch (Exception ex){
            System.out.println(ex.getMessage());
        }

        System.out.println(vertexShaderId);
    }

    public void createFragmentShader(String shaderCode) throws Exception{
        fragmentShaderId = createShader(shaderCode,GL_FRAGMENT_SHADER);
        System.out.println(vertexShaderId);
    }

    protected int createShader(String shaderCode, int shaderType) throws Exception {
        int shaderId = glCreateShader(shaderType);
        if (shaderId == 0){
            throw new Exception("Error creating shader. Type: " + shaderType);
        }

        glShaderSource(shaderId,shaderCode);
        glCompileShader(shaderId);

        if(glGetShaderi(shaderId,GL_COMPILE_STATUS) == 0){
            throw new Exception("Error compiling Shader code: " + glGetShaderInfoLog(shaderId,1024));
        }

        glAttachShader(programId,shaderId);

        return shaderId;
    }

    public void link() throws Exception{
        glLinkProgram(programId);

        if (glGetProgrami(programId,GL_LINK_STATUS) == 0)
            throw new Exception("Could not load program" + glGetProgramInfoLog(programId, 1024));

        if (vertexShaderId != 0)
            glDetachShader(programId,vertexShaderId);

        if (fragmentShaderId != 0)
            glDetachShader(programId,fragmentShaderId);


        glValidateProgram(programId);
        if (glGetProgrami(programId,GL_VALIDATE_STATUS) == 0)
            System.out.println("shader is not validated: " + glGetProgramInfoLog(programId,1024));
    }

    public void bind(){
        glUseProgram(programId);
    }

    public void unbind(){
        glUseProgram(0);
    }

    public void cleanUp(){
        unbind();

        if (programId != 0)
            glDeleteProgram(programId);
    }

    public int getProgramId() {
        return programId;
    }
}
